@@ -4,8 +4,9 @@ use 5.006;
 use strict;
 use warnings;
 use Class::Tiny;
+use Types::TypeTiny ();
 
-our $VERSION = '0.000013';
+our $VERSION = '0.000015';
 
 # Docs {{{1
 
@@ -174,7 +175,14 @@ sub import {
 # Returns two coderefs: checker and get_message.
 sub _get_constraint_sub {
     my ($type_name, $type) = @_;
-    my ($checker, $get_message);
+
+    # Get type's name, if any
+    my $name = eval { $type->can('name') || $type->can('description') };
+    $name = $type->$name() if $name;
+
+    # Set default message
+    $name = $type_name unless $name;
+    my $default_get_message = sub { "Value is not a $name" };
 
     # Handle the custom-constraint format
     if(ref $type eq 'ARRAY') {
@@ -185,60 +193,33 @@ sub _get_constraint_sub {
         return @$type;
     }
 
-    # Get type's name, if any
-    my $name = eval { $type->can('name') || $type->can('description') };
-    $name = $type->$name() if $name;
+    # Handle MooX::Types::MooseLike as well as other types of coderef.
+    # $orig_coderef may indicate failure by dying or returning 0 ---
+    # there's no way to know.  We assume that:
+    #   - Express undef is success (e.g., MooX::Types::MooseLike)
+    #   - die() is failure (ditto)
+    #   - Returning a defined value is a success/failure indication
+    #     (as expected by Types::TypeTiny::to_TypeTiny(\&)).
+    if(ref $type eq 'CODE') {
+        my $orig_coderef = $type;
+        my $new_type = sub {
+            local $@;
+            my $is_ok = eval { $orig_coderef->(@_) };
+            if($@) {
+                $is_ok = 0;                     # die() => failure
+            } elsif(!defined $is_ok) {
+                $is_ok = 1;                     # undef => success
+            }
+            return $is_ok;
+        };
+        $type = $new_type;
+    }
 
-    # Set default message
-    $name = $type_name unless $name;
-    $get_message = sub { "Value is not a $name" };
+    my $impl = Types::TypeTiny::to_TypeTiny($type);
+    die "I couldn't understand the constraint for $type_name"
+        unless ref $impl eq 'Type::Tiny';
 
-    # Try the different types of constraints we know about
-    DONE: {
-
-        if (ref($type) eq 'CODE') { # MooX::Types::MooseLike
-            $checker = sub { eval { $type->(@_); 1 } };
-                # In profiling, seems to be about on par with `&$type;`.
-            last DONE;
-        }
-
-        die "I don't know how to handle non-reference constraint $type_name"
-            unless ref $type;   # All the rest of the checks use $type->can()
-
-        if ( my $method = eval { $type->can('compiled_check') }) { # Type::Tiny
-            $checker = $type->$method();
-            $get_message = sub { $type->get_message($_[0]) };
-            last DONE;
-        }
-
-        if (my $method = eval { $type->can('inline_check') || $type->can('_inline_check') }) { # Specio::Constraint::Simple
-            $checker = eval {
-                my $text = $type->$method('$value');
-                    # $method() will fail if type cannot be inlined.
-                local $SIG{__WARN__} = sub { };     # Eval failure => silent
-                eval "sub { my \$value = shift; $text }"
-            };
-            last DONE if $checker;
-        }
-
-        if (my $method = eval { $type->can('check') } ) { # Moose, Mouse
-            $checker = sub { unshift @_, $type; goto &$method; };
-                # Like $type->check(@_), but profiles as much faster on my
-                # test system.
-            last DONE;
-        }
-
-        if(eval { $type->can('value_is_valid') }) { # Specio::Constraint::Simple
-            $checker = sub { $type->value_is_valid(@_) };
-            last DONE;
-        }
-
-    } #DONE
-
-    die "$type_name: I don't know how to use type " . ref($type)
-        unless $checker;
-
-    return ($checker, $get_message);
+    return ( $impl->compiled_check(), sub { $impl->get_message($_[0]) } );
 } #_get_constraint_sub()
 
 # _make_accessor($name, \&checker, \&get_message): Make an accessor.
